class ROT13():
    @staticmethod
    def encode(char: str):
        return char.translate(bytes.maketrans(b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz', b'NOPQRSTUVWXYZABCDEFGHIJKLMnopqrstuvwxyzabcdefghijklm'))

    @staticmethod
    def decode(char: str):
        return char.translate(bytes.maketrans(b'NOPQRSTUVWXYZABCDEFGHIJKLMnopqrstuvwxyzabcdefghijklm', b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'))


class AtBash():
    @staticmethod
    def encode(char):
        AtBash = ""

        for letters in char:
            if letters.isupper():
                AtBash += chr((ord('Z') + ord('A')) - ord(letters))

            elif letters.islower():
                AtBash += chr(ord('z') + ord('a') - ord(letters))

            else:
                AtBash += letters

        return AtBash

    @staticmethod
    def decode(char: str):
        AtBash = ""

        for letters in char:
            if letters.isupper():
                Code = ord('Z') + ord('A')
                AtBash += chr(Code - ord(letters))

            elif letters.islower():
                Code = ord('z') + ord('a')
                AtBash += chr(Code - ord(letters))

            else:
                AtBash += letters

        return AtBash


class Bacon():
    BaconDict = {'A': 'aaaaa', 'B': 'aaaab', 'C': 'aaaba', 'D': 'aaabb', 'E': 'aabaa',
                 'F': 'aabab', 'G': 'aabba', 'H': 'aabbb', 'I': 'abaaa', 'J': 'abaab',
                 'K': 'ababa', 'L': 'ababb', 'M': 'abbaa', 'N': 'abbab', 'O': 'abbba',
                 'P': 'abbbb', 'Q': 'baaaa', 'R': 'baaab', 'S': 'baaba', 'T': 'baabb',
                 'U': 'babaa', 'V': 'babab', 'W': 'babba', 'X': 'babbb', 'Y': 'bbaaa',
                 'Z': 'bbaab'}
 

    def encode(self, char: str):
        return char.upper().translate(str.maketrans(self.BaconDict))

    
    def decode(self, char: str):
        BaconText = ''
        i = 0
        while True:
            if(i < len(char)-4):
                sets = char[i:i + 5]
                if sets[0] != ' ':
                    BaconText += list(self.BaconDict.keys()
                                      )[list(self.BaconDict.values()).index(sets)].lower()
                    i += 5
                else:
                    BaconText += ' '
                    i += 1
            else:
                break
        return BaconText

    
class A1Z26():
    def encode(char: str):
        A1Z26 = []

        for i in char:
            if i.isalpha():
                A1Z26.append(str(ord(i.lower()) - 96))

            else:
                A1Z26.append(i)

        for i in range(len(A1Z26)):
            if A1Z26[i] == " ":
                A1Z26[i] = ""

        return " ".join(A1Z26)

    def decode(char: str):
        A1Z26 = []

        for i in char.split(" "):
            if i != "":
                A1Z26.append(chr(int(i) + 96))

            else:
                A1Z26.append(" ")

        return "".join(A1Z26)


class Caesar():

    def encode(char: str, shift: int = 3):
        caesar = ""

        for i in range(len(char)):
            if char[i].isupper():
                caesar += chr((ord(char[i]) + shift - 65) % 26 + 65)

            elif char[i].islower():
                caesar += chr((ord(char[i]) + shift - 97) % 26 + 97)
            else:
                caesar += char[i]

        return caesar

    def decode(char: str, shift: int = 3):
        caesar = ""

        for i in range(len(char)):
            if char[i].isupper():
                caesar += chr((ord(char[i]) - shift - 65) % 26 + 65)

            elif char[i].islower():
                caesar += chr((ord(char[i]) - shift - 97) % 26 + 97)
            else:
                caesar += char[i]

        return caesar


class MorseCode: 
    MorseCodeDict = {   'A':'.-', 'B':'-...','C':'-.-.', 
                        'D':'-..', 'E':'.', 'F':'..-.', 
                        'G':'--.', 'H':'....', 'I':'..', 
                        'J':'.---', 'K':'-.-', 'L':'.-..', 
                        'M':'--', 'N':'-.', 'O':'---', 
                        'P':'.--.', 'Q':'--.-', 'R':'.-.', 
                        'S':'...', 'T':'-', 'U':'..-', 
                        'V':'...-', 'W':'.--','X':'-..-', 
                        'Y':'-.--', 'Z':'--..', '1':'.----', 
                        '2':'..---', '3':'...--', '4':'....-', 
                        '5':'.....', '6':'-....', '7':'--...', 
                        '8':'---..', '9':'----.', '0':'-----', 
                        ', ':'--..--', '.':'.-.-.-', '?':'..--..', 
                        '/':'-..-.', '-':'-....-', '(':'-.--.', ')':'-.--.-',
                        ' ' : '/'    
                    }

    inverseMorseCodeDict = dict((v, k) for (k, v) in MorseCodeDict.items())

 
    def encode(self, char: str):
        MorseCodeText = ""

        for i in char.upper():
            if i != " ":
                MorseCodeText += self.MorseCodeDict[i] + " "

            else:
                MorseCodeText += "/ "

        return MorseCodeText

    def decode(self, char: str):
        char += ' '
        decipher = ''
        containIT = ''
        
        for i in char:
            if (i != ' '):
                counter = 0
                containIT += i

            else:
                counter += 1
                if counter == 2:
                    decipher += ' '

                else:
                    decipher += list(self.MorseCodeDict.keys())[list(self.MorseCodeDict.values()).index(containIT)]
                    containIT = ''
    
        return decipher


from pyclbr import Function
from colorama import Fore

class NoSpaceError(Exception):
    """Raised when there is no space in the given Text"""
    def __init__(self, text):
        self.text = text
        self.message = f"There were no spaces identified in the Variable's Value | Value: \"{self.text}\""
        super().__init__(self.message)

class CharLengthShort(Exception):
    """Raised when the length of the'Char' doesn't matches with the base """
    def __init__(self, Class, Function, CharLen: int):
        self.message = f"The 'char' received a very short length. The {Class}'s {Function}: Requires a length of {CharLen}."
        super().__init__(self.message)


def CustomValueError(Class: str, Func: str, Param):
    return f"{Fore.RED}[x] " f"{Fore.BLUE}[{Class}: {Func}] - " f"{Fore.RED}'{Param}' is not a positive number. {Fore.RESET}"




import urllib.parse


class Binary:
    @staticmethod
    def encode(char: str):
        return " ".join(format(ord(i), '08b') for i in char.strip())

    @staticmethod
    def decode(char: int):
        return "".join(chr(int(i, 2)) for i in char.strip().split(" "))


class Hexadecimal:
    @staticmethod
    def encode(char: str):
        return " ".join(format(ord(i), 'x') for i in char)

    @staticmethod
    def decode(char: str):
        return "".join(chr(int(i, 16)) for i in char.strip().split(" "))


class Octal:
    @staticmethod
    def encode(char: str):
        return " ".join(format(ord(i), 'o') for i in char)

    @staticmethod
    def decode(char: str):
        return "".join(chr(int(i, 8)) for i in char.strip().split(" "))


class Integer:
    @staticmethod
    def encode(char: str):
        return " ".join(str(ord(i)) for i in char)

    @staticmethod
    def decode(char):
        return "".join(chr(int(i)) for i in str(char).split(" "))


class UrlEncoding:
    @staticmethod
    def encode(char: str):
        return urllib.parse.quote(char)

    @staticmethod
    def decode(char: str):
        return urllib.parse.unquote(char)


class UnicodePoint:
    @staticmethod
    def encode(char):
        return " ".join([str(hex(ord(i))) for i in char])

    @staticmethod
    def decode(char):
        ch = []

        for uni in str(char).split(" "):
            ch.append(chr(int(uni, 16)))

        return "".join(ch)





import setuptools
from setuptools import setup


classifiers = [
  'Development Status :: 5 - Production/Stable',
  'Intended Audience :: Developers',
  'Operating System :: OS Independent',
  'License :: OSI Approved :: MIT License',
  'Natural Language :: English',
  'Programming Language :: Python :: 3'
]

setup(
  name = 'hashily',
  version = '0.0.3',
  description = 'A module that allows you to encode and decode text in numerous ciphers.',
  long_description = open('README.md').read(),
  long_description_content_type = "text/markdown",
  url = "https://github.com/MystYT-21/hashily", 
  project_urls = {
   "Issue tracker": "https://github.com/MystYT-21/hashily/issues",
                 },

  author = 'myst.67',
  author_email = 'saidulalamirfan3245@gmail.com',
  license = 'MIT', 
  classifiers= classifiers,
  keywords = ["encryption", "utils", "encode", "ciphers", "decode"], 
  packages = setuptools.find_packages(),
)


import random
import string

def password(length: int = 16):
    chars = list(string.ascii_letters + string.digits + "!@#$%^&*()")
    random.shuffle(chars)

    foobar = []

    try:
        if length > 16:
            return "The Password can be no longer than 16 and no shorter than eight characters"

        else:
            for i in range(length):
                foobar.append(random.choice(chars))

            return "".join(foobar)

    except:
        pass


def token(length: int = 59):
    if length > 59:
        raise

    else:
        chars = list(string.ascii_letters + "-._" + string.digits)
        random.shuffle(chars)

        foobar = []

        for i in range(length):
            foobar.append(random.choice(chars))

        random.shuffle(foobar)

        return "".join(foobar)


def reverse(text: str):
    return text[::-1]


def titlelize(text: str):
    return " ".join(i.strip().lower().capitalize() for i in text.split())
